import contextvars
import os
from hashlib import md5
from pathlib import Path
from random import getrandbits
from typing import List, Union

from graphviz import Digraph

__version__ = "0.1.0"

# Global context for a diagrams and a cluster.
#
# Theses global contexts are for letting the clusters and nodes know
# where context they are belong to. So the all clusters and nodes does
# not need to specify the current diagrams or cluster via parameters.
__diagram = contextvars.ContextVar("diagrams")
__cluster = contextvars.ContextVar("cluster")


def getdiagram():
    try:
        return __diagram.get()
    except LookupError:
        return None


def setdiagram(diagram):
    __diagram.set(diagram)


def getcluster():
    try:
        return __cluster.get()
    except LookupError:
        return None


def setcluster(cluster):
    __cluster.set(cluster)


class Diagram:
    __directions = ("TB", "BT", "LR", "RL")
    __outformats = ("png", "jpg", "svg", "pdf")

    # fmt: off
    _default_graph_attrs = {
        "pad": "2.0",
        "splines": "ortho",
        "nodesep": "0.60",
        "ranksep": "0.75",
        "fontname": "Sans-Serif",
        "fontsize": "15",
        "fontcolor": "#2D3436",
    }
    _default_node_attrs = {
        "shape": "box",
        "style": "rounded",
        "fixedsize": "true",
        "width": "1.4",
        "height": "1.4",
        "labelloc": "b",
        "imagepos": "tc",
        "imagescale": "true",
        "fontname": "Sans-Serif",
        "fontsize": "13",
        "fontcolor": "#2D3436",
    }
    _default_edge_attrs = {
        "color": "#7B8894",
    }

    # fmt: on

    # TODO: Label position option
    # TODO: Save directory option (filename + directory?)
    def __init__(self, name: str = "", direction: str = "LR", outformat: str = "png", show: bool = True):
        """Diagram represents a global diagrams context.

        :param name: Diagram name. It will be used for output filename.
        :param direction: Data flow direction. Default is 'left to right'.
        :param outformat: Output file format. Default is 'png'.
        :param show: Open generated image after save if true, just only save otherwise.
        """
        self.name = name

        self.filename = "_".join(self.name.split()).lower()
        self.dot = Digraph(self.name, filename=self.filename)

        # Set attributes.
        for k, v in self._default_graph_attrs.items():
            self.dot.graph_attr[k] = v
        self.dot.graph_attr["label"] = self.name
        for k, v in self._default_node_attrs.items():
            self.dot.node_attr[k] = v
        for k, v in self._default_edge_attrs.items():
            self.dot.edge_attr[k] = v

        if not self._validate_direction(direction):
            raise ValueError(f'"{direction}" is not a valid direction')
        self.dot.graph_attr["rankdir"] = direction

        if not self._validate_outformat(outformat):
            raise ValueError(f'"{outformat}" is not a valid output format')
        self.outformat = outformat

        self.show = show

    def __enter__(self):
        setdiagram(self)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.render()
        # Remove the graphviz file leaving only the image.
        os.remove(self.filename)
        setdiagram(None)

    def _validate_direction(self, direction: str) -> bool:
        direction = direction.upper()
        for v in self.__directions:
            if v == direction:
                return True
        return False

    def _validate_outformat(self, outformat: str) -> bool:
        outformat = outformat.lower()
        for v in self.__outformats:
            if v == outformat:
                return True
        return False

    def node(self, hashid: str, label: str, **attrs) -> None:
        """Create a new node."""
        self.dot.node(hashid, label=label, **attrs)

    def connect(self, node: "Node", node2: "Node", directed=True) -> None:
        """Connect the two Nodes."""
        attrs = {"dir": "none"} if not directed else {}
        self.dot.edge(node.hashid, node2.hashid, **attrs)

    def reverse(self, node: "Node", node2: "Node", directed=True) -> None:
        """Connect the two Nodes in reverse direction."""
        attrs = {"dir": "none"} if not directed else {"dir": "back"}
        self.dot.edge(node.hashid, node2.hashid, **attrs)

    def subgraph(self, dot: Digraph) -> None:
        """Create a subgraph for clustering"""
        self.dot.subgraph(dot)

    def render(self) -> None:
        self.dot.render(format=self.outformat, view=self.show)


class Cluster:
    __directions = ("TB", "BT", "LR", "RL")
    __bgcolors = ("#E5F5FD", "#EBF3E7", "#ECE8F6", "#FDF7E3")

    # fmt: off
    _default_graph_attrs = {
        "shape": "box",
        "style": "rounded",
        "labeljust": "l",
        "pencolor": "#AEB6BE",
        "fontname": "Sans-Serif",
        "fontsize": "12",
    }

    # fmt: on

    # FIXME:
    #  Cluster direction does not work now. Graphviz couldn't render
    #  correctly for a subgraph that has a different rank direction.
    def __init__(self, label: str = "cluster", direction: str = "LR"):
        """Cluster represents a cluster context.

        :param label: Cluster label.
        :param direction: Data flow direction. Default is 'left to right'.
        """
        self.label = label
        self.name = "cluster_" + self.label

        self.dot = Digraph(self.name)

        # Set attributes.
        for k, v in self._default_graph_attrs.items():
            self.dot.graph_attr[k] = v
        self.dot.graph_attr["label"] = self.label

        if not self._validate_direction(direction):
            raise ValueError(f'"{direction}" is not a valid direction')
        self.dot.graph_attr["rankdir"] = direction

        # Node must be belong to a diagrams.
        self._diagram = getdiagram()
        if self._diagram is None:
            raise EnvironmentError("Global diagrams context not set up")
        self._parent = getcluster()

        # Set cluster depth for distinguishing the background color
        self.depth = self._parent.depth + 1 if self._parent else 0
        coloridx = self.depth % len(self.__bgcolors)
        self.dot.graph_attr["bgcolor"] = self.__bgcolors[coloridx]

    def __enter__(self):
        setcluster(self)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self._parent:
            self._parent.subgraph(self.dot)
        else:
            self._diagram.subgraph(self.dot)
        setcluster(self._parent)

    def _validate_direction(self, direction: str):
        direction = direction.upper()
        for v in self.__directions:
            if v == direction:
                return True
        return False

    def node(self, hashid: str, label: str, **attrs) -> None:
        """Create a new node in the cluster."""
        self.dot.node(hashid, label=label, **attrs)

    def subgraph(self, dot: Digraph) -> None:
        self.dot.subgraph(dot)


class Node:
    """Node represents a node for a specific backend service."""

    _provider = None
    _type = None

    _icon_dir = None
    _icon = None

    def __init__(self, label: str = ""):
        """Node represents a system component.

        :param label: Node label.
        """
        # Generates a hash for identifying a node.
        self._hash = self._rand_hash()
        self.label = label

        # fmt: off
        # If a node has an icon, increase the height slightly to avoid
        # that label being spanned between icon image and white space.
        self.attrs = {
            "shape": "none",
            "height": "1.6",
            "image": self._load_icon(),
        } if self._icon else {}
        # fmt: on

        # Node must be belong to a diagrams.
        self._diagram = getdiagram()
        if self._diagram is None:
            raise EnvironmentError("Global diagrams context not set up")
        self._cluster = getcluster()

        # If a node is in the cluster context, add it to cluster.
        if self._cluster:
            self._cluster.node(self._hash, self.label, **self.attrs)
        else:
            self._diagram.node(self._hash, self.label, **self.attrs)

    def __repr__(self):
        _name = self.__class__.__name__
        return f"<{self._provider}.{self._type}.{_name}>"

    def __sub__(self, other: Union["Node", List["Node"]]):
        """Implement Self - Node and Self - [Nodes]"""
        if not isinstance(other, list):
            return self.connect(other, directed=False)
        for node in other:
            self.connect(node, directed=False)
        return other

    def __rsub__(self, other: List["Node"]):
        """
        Called for [Nodes] - Self because list of Nodes don't have
        __sub__ operators.
        """
        self.__sub__(other)
        return self

    def __rshift__(self, other: Union["Node", List["Node"]]):
        """Implements Self >> Node and Self >> [Nodes]."""
        if not isinstance(other, list):
            return self.connect(other)
        for node in other:
            self.connect(node)
        return other

    def __lshift__(self, other: Union["Node", List["Node"]]):
        """Implements Self << Node and Self << [Nodes]."""
        if not isinstance(other, list):
            return self.reverse(other)
        for node in other:
            self.reverse(node)
        return other

    def __rrshift__(self, other: List["Node"]):
        """
        Called for [Nodes] >> Self because list of Nodes don't have
        __rshift__ operators.
        """
        for node in other:
            node.connect(self)
        return self

    def __rlshift__(self, other: List["Node"]):
        """
        Called for [Nodes] << Self because list of Nodes don't have
        __lshift__ operators.
        """
        for node in other:
            node.reverse(self)
        return self

    @property
    def hashid(self):
        return self._hash

    # TODO: option for adding flow description to the connection edge
    def connect(self, node: "Node", directed=True):
        """Connect to other node.

        :param node: Other node instance.
        :param directed: Whether the flow is directed or not.
        :return: Connected node.
        """
        if not isinstance(node, Node):
            ValueError(f"{node} is not a valid Node")
        # An edge must be added on the global diagrams, not a cluster.
        self._diagram.connect(self, node, directed)
        return node

    def reverse(self, node: "Node", directed=True):
        """Connect to other node in reverse direction.

        :param node: Other node instance.
        :param directed: Whether the flow is directed or not.
        :return: Connected node.
        """
        if not isinstance(node, Node):
            ValueError(f"{node} is not a valid Node")
        # An edge must be added on the global diagrams, not a cluster.
        self._diagram.reverse(self, node, directed)
        return node

    @staticmethod
    def _rand_hash():
        return md5(getrandbits(64).to_bytes(64, "big")).hexdigest()

    def _load_icon(self):
        basedir = Path(os.path.abspath(os.path.dirname(__file__)))
        return os.path.join(basedir.parent, self._icon_dir, self._icon)


Group = Cluster



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _Azure


class _Database(_Azure):
    _type = "database"
    _icon_dir = "resources/azure/database"


class BlobStorage(_Database):
    _icon = "blob-storage.png"


class CacheForRedis(_Database):
    _icon = "cache-for-redis.png"


class CosmosDb(_Database):
    _icon = "cosmos-db.png"


class DataLake(_Database):
    _icon = "data-lake.png"


class DatabaseForMariadbServers(_Database):
    _icon = "database-for-mariadb-servers.png"


class DatabaseForMysqlServers(_Database):
    _icon = "database-for-mysql-servers.png"


class DatabaseForPostgresqlServers(_Database):
    _icon = "database-for-postgresql-servers.png"


class ElasticDatabasePools(_Database):
    _icon = "elastic-database-pools.png"


class ElasticJobAgents(_Database):
    _icon = "elastic-job-agents.png"


class ManagedDatabases(_Database):
    _icon = "managed-databases.png"


class SQLDatabases(_Database):
    _icon = "sql-databases.png"


class SQLDatawarehouse(_Database):
    _icon = "sql-datawarehouse.png"


class SQLManagedInstances(_Database):
    _icon = "sql-managed-instances.png"


class SQLServerStretchDatabases(_Database):
    _icon = "sql-server-stretch-databases.png"


class SQLServers(_Database):
    _icon = "sql-servers.png"


class VirtualClusters(_Database):
    _icon = "virtual-clusters.png"


class VirtualDatacenter(_Database):
    _icon = "virtual-datacenter.png"


# Aliases



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _Azure


class _Analytics(_Azure):
    _type = "analytics"
    _icon_dir = "resources/azure/analytics"


class AnalysisServices(_Analytics):
    _icon = "analysis-services.png"


class DataExplorerClusters(_Analytics):
    _icon = "data-explorer-clusters.png"


class DataFactories(_Analytics):
    _icon = "data-factories.png"


class DataLakeAnalytics(_Analytics):
    _icon = "data-lake-analytics.png"


class DataLakeStoreGen1(_Analytics):
    _icon = "data-lake-store-gen1.png"


class Databricks(_Analytics):
    _icon = "databricks.png"


class EventHubClusters(_Analytics):
    _icon = "event-hub-clusters.png"


class EventHubs(_Analytics):
    _icon = "event-hubs.png"


class Hdinsightclusters(_Analytics):
    _icon = "hdinsightclusters.png"


class LogAnalyticsWorkspaces(_Analytics):
    _icon = "log-analytics-workspaces.png"


class StreamAnalyticsJobs(_Analytics):
    _icon = "stream-analytics-jobs.png"


# Aliases



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Security(_AWS):
    _type = "security"
    _icon_dir = "resources/aws/security"


class Artifact(_Security):
    _icon = "artifact.png"


class CertificateManager(_Security):
    _icon = "certificate-manager.png"


class CloudDirectory(_Security):
    _icon = "cloud-directory.png"


class Cloudhsm(_Security):
    _icon = "cloudhsm.png"


class Cognito(_Security):
    _icon = "cognito.png"


class DirectoryService(_Security):
    _icon = "directory-service.png"


class FirewallManager(_Security):
    _icon = "firewall-manager.png"


class Guardduty(_Security):
    _icon = "guardduty.png"


class IdentityAndAccessManagementIam(_Security):
    _icon = "identity-and-access-management-iam.png"


class Inspector(_Security):
    _icon = "inspector.png"


class KeyManagementService(_Security):
    _icon = "key-management-service.png"


class Macie(_Security):
    _icon = "macie.png"


class ResourceAccessManager(_Security):
    _icon = "resource-access-manager.png"


class SecretsManager(_Security):
    _icon = "secrets-manager.png"


class SecurityHub(_Security):
    _icon = "security-hub.png"


class SecurityIdentityAndCompliance(_Security):
    _icon = "security-identity-and-compliance.png"


class Shield(_Security):
    _icon = "shield.png"


class SingleSignOn(_Security):
    _icon = "single-sign-on.png"


class WAF(_Security):
    _icon = "waf.png"


# Aliases

ACM = CertificateManager
CloudHSM = Cloudhsm
DS = DirectoryService
FMS = FirewallManager
IAM = IdentityAndAccessManagementIam
KMS = KeyManagementService
RAM = ResourceAccessManager



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _GCP


class _Database(_GCP):
    _type = "database"
    _icon_dir = "resources/gcp/database"


class Bigtable(_Database):
    _icon = "bigtable.png"


class Datastore(_Database):
    _icon = "datastore.png"


class Firestore(_Database):
    _icon = "firestore.png"


class Memorystore(_Database):
    _icon = "memorystore.png"


class Spanner(_Database):
    _icon = "spanner.png"


class SQL(_Database):
    _icon = "sql.png"


# Aliases

BigTable = Bigtable



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Network(_AWS):
    _type = "network"
    _icon_dir = "resources/aws/network"


class APIGateway(_Network):
    _icon = "api-gateway.png"


class AppMesh(_Network):
    _icon = "app-mesh.png"


class ClientVpn(_Network):
    _icon = "client-vpn.png"


class CloudMap(_Network):
    _icon = "cloud-map.png"


class Cloudfront(_Network):
    _icon = "cloudfront.png"


class DirectConnect(_Network):
    _icon = "direct-connect.png"


class ElasticLoadBalancing(_Network):
    _icon = "elastic-load-balancing.png"


class GlobalAccelerator(_Network):
    _icon = "global-accelerator.png"


class NetworkingAndContentDelivery(_Network):
    _icon = "networking-and-content-delivery.png"


class Privatelink(_Network):
    _icon = "privatelink.png"


class Route53(_Network):
    _icon = "route-53.png"


class SiteToSiteVpn(_Network):
    _icon = "site-to-site-vpn.png"


class TransitGateway(_Network):
    _icon = "transit-gateway.png"


class VPC(_Network):
    _icon = "vpc.png"


# Aliases

CF = Cloudfront
ELB = ElasticLoadBalancing
GAX = GlobalAccelerator



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Compute(_AWS):
    _type = "compute"
    _icon_dir = "resources/aws/compute"


class ApplicationAutoScaling(_Compute):
    _icon = "application-auto-scaling.png"


class Batch(_Compute):
    _icon = "batch.png"


class Compute(_Compute):
    _icon = "compute.png"


class EC2ContainerRegistry(_Compute):
    _icon = "ec2-container-registry.png"


class EC2(_Compute):
    _icon = "ec2.png"


class ElasticBeanstalk(_Compute):
    _icon = "elastic-beanstalk.png"


class ElasticContainerService(_Compute):
    _icon = "elastic-container-service.png"


class ElasticKubernetesService(_Compute):
    _icon = "elastic-kubernetes-service.png"


class Fargate(_Compute):
    _icon = "fargate.png"


class Lambda(_Compute):
    _icon = "lambda.png"


class Lightsail(_Compute):
    _icon = "lightsail.png"


class Outposts(_Compute):
    _icon = "outposts.png"


class ServerlessApplicationRepository(_Compute):
    _icon = "serverless-application-repository.png"


class ThinkboxDeadline(_Compute):
    _icon = "thinkbox-deadline.png"


class ThinkboxDraft(_Compute):
    _icon = "thinkbox-draft.png"


class ThinkboxFrost(_Compute):
    _icon = "thinkbox-frost.png"


class ThinkboxKrakatoa(_Compute):
    _icon = "thinkbox-krakatoa.png"


class ThinkboxSequoia(_Compute):
    _icon = "thinkbox-sequoia.png"


class ThinkboxStoke(_Compute):
    _icon = "thinkbox-stoke.png"


class ThinkboxXmesh(_Compute):
    _icon = "thinkbox-xmesh.png"


class VmwareCloudOnAWS(_Compute):
    _icon = "vmware-cloud-on-aws.png"


# Aliases

AutoScaling = ApplicationAutoScaling
ECR = EC2ContainerRegistry
EB = ElasticBeanstalk
ECS = ElasticContainerService
EKS = ElasticKubernetesService
SAR = ServerlessApplicationRepository



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _Azure


class _Migration(_Azure):
    _type = "migration"
    _icon_dir = "resources/azure/migration"


class DatabaseMigrationServices(_Migration):
    _icon = "database-migration-services.png"


class MigrationProjects(_Migration):
    _icon = "migration-projects.png"


class RecoveryServicesVaults(_Migration):
    _icon = "recovery-services-vaults.png"


# Aliases



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _ML(_AWS):
    _type = "ml"
    _icon_dir = "resources/aws/ml"


class ApacheMxnetOnAWS(_ML):
    _icon = "apache-mxnet-on-aws.png"


class Comprehend(_ML):
    _icon = "comprehend.png"


class DeepLearningAmis(_ML):
    _icon = "deep-learning-amis.png"


class DeepLearningContainers(_ML):
    _icon = "deep-learning-containers.png"


class Deeplens(_ML):
    _icon = "deeplens.png"


class Deepracer(_ML):
    _icon = "deepracer.png"


class ElasticInference(_ML):
    _icon = "elastic-inference.png"


class Forecast(_ML):
    _icon = "forecast.png"


class Lex(_ML):
    _icon = "lex.png"


class MachineLearning(_ML):
    _icon = "machine-learning.png"


class Personalize(_ML):
    _icon = "personalize.png"


class Polly(_ML):
    _icon = "polly.png"


class Rekognition(_ML):
    _icon = "rekognition.png"


class SagemakerGroundTruth(_ML):
    _icon = "sagemaker-ground-truth.png"


class Sagemaker(_ML):
    _icon = "sagemaker.png"


class TensorflowOnAWS(_ML):
    _icon = "tensorflow-on-aws.png"


class Textract(_ML):
    _icon = "textract.png"


class Transcribe(_ML):
    _icon = "transcribe.png"


class Translate(_ML):
    _icon = "translate.png"


# Aliases

DLC = DeepLearningContainers



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _GCP


class _ML(_GCP):
    _type = "ml"
    _icon_dir = "resources/gcp/ml"


class AdvancedSolutionsLab(_ML):
    _icon = "advanced-solutions-lab.png"


class AIHub(_ML):
    _icon = "ai-hub.png"


class AIPlatformDataLabelingService(_ML):
    _icon = "ai-platform-data-labeling-service.png"


class AIPlatform(_ML):
    _icon = "ai-platform.png"


class AutomlNaturalLanguage(_ML):
    _icon = "automl-natural-language.png"


class AutomlTables(_ML):
    _icon = "automl-tables.png"


class AutomlTranslation(_ML):
    _icon = "automl-translation.png"


class AutomlVideoIntelligence(_ML):
    _icon = "automl-video-intelligence.png"


class AutomlVision(_ML):
    _icon = "automl-vision.png"


class Automl(_ML):
    _icon = "automl.png"


class DialogFlowEnterpriseEdition(_ML):
    _icon = "dialog-flow-enterprise-edition.png"


class InferenceAPI(_ML):
    _icon = "inference-api.png"


class JobsAPI(_ML):
    _icon = "jobs-api.png"


class NaturalLanguageAPI(_ML):
    _icon = "natural-language-api.png"


class RecommendationsAI(_ML):
    _icon = "recommendations-ai.png"


class SpeechToText(_ML):
    _icon = "speech-to-text.png"


class TextToSpeech(_ML):
    _icon = "text-to-speech.png"


class TPU(_ML):
    _icon = "tpu.png"


class TranslationAPI(_ML):
    _icon = "translation-api.png"


class VideoIntelligenceAPI(_ML):
    _icon = "video-intelligence-api.png"


class VisionAPI(_ML):
    _icon = "vision-api.png"


# Aliases

AutoML = Automl
NLAPI = NaturalLanguageAPI
STT = SpeechToText
TTS = TextToSpeech



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Iot(_AWS):
    _type = "iot"
    _icon_dir = "resources/aws/iot"


class Freertos(_Iot):
    _icon = "freertos.png"


class InternetOfThings(_Iot):
    _icon = "internet-of-things.png"


class Iot1Click(_Iot):
    _icon = "iot-1-click.png"


class IotAnalytics(_Iot):
    _icon = "iot-analytics.png"


class IotButton(_Iot):
    _icon = "iot-button.png"


class IotCore(_Iot):
    _icon = "iot-core.png"


class IotDeviceDefender(_Iot):
    _icon = "iot-device-defender.png"


class IotDeviceManagement(_Iot):
    _icon = "iot-device-management.png"


class IotEvents(_Iot):
    _icon = "iot-events.png"


class IotGreengrass(_Iot):
    _icon = "iot-greengrass.png"


class IotSitewise(_Iot):
    _icon = "iot-sitewise.png"


class IotThingsGraph(_Iot):
    _icon = "iot-things-graph.png"


# Aliases

FreeRTOS = Freertos



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _GCP


class _Analytics(_GCP):
    _type = "analytics"
    _icon_dir = "resources/gcp/analytics"


class Bigquery(_Analytics):
    _icon = "bigquery.png"


class Composer(_Analytics):
    _icon = "composer.png"


class DataCatalog(_Analytics):
    _icon = "data-catalog.png"


class DataFusion(_Analytics):
    _icon = "data-fusion.png"


class Dataflow(_Analytics):
    _icon = "dataflow.png"


class Datalab(_Analytics):
    _icon = "datalab.png"


class Dataprep(_Analytics):
    _icon = "dataprep.png"


class Dataproc(_Analytics):
    _icon = "dataproc.png"


class Genomics(_Analytics):
    _icon = "genomics.png"


class Pubsub(_Analytics):
    _icon = "pubsub.png"


# Aliases

BigQuery = Bigquery
PubSub = Pubsub



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Database(_AWS):
    _type = "database"
    _icon_dir = "resources/aws/database"


class Aurora(_Database):
    _icon = "aurora.png"


class DatabaseMigrationService(_Database):
    _icon = "database-migration-service.png"


class Database(_Database):
    _icon = "database.png"


class DocumentdbMongodbCompatibility(_Database):
    _icon = "documentdb-mongodb-compatibility.png"


class Dynamodb(_Database):
    _icon = "dynamodb.png"


class Elasticache(_Database):
    _icon = "elasticache.png"


class Neptune(_Database):
    _icon = "neptune.png"


class QuantumLedgerDatabaseQldb(_Database):
    _icon = "quantum-ledger-database-qldb.png"


class RDSOnVmware(_Database):
    _icon = "rds-on-vmware.png"


class RDS(_Database):
    _icon = "rds.png"


class Redshift(_Database):
    _icon = "redshift.png"


class Timestream(_Database):
    _icon = "timestream.png"


# Aliases

DMS = DatabaseMigrationService
DocumentDB = DocumentdbMongodbCompatibility
DB = Database
DDB = Dynamodb
ElastiCache = Elasticache
QLDB = QuantumLedgerDatabaseQldb



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _GCP


class _Iot(_GCP):
    _type = "iot"
    _icon_dir = "resources/gcp/iot"


class IotCore(_Iot):
    _icon = "iot-core.png"


# Aliases



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _Azure


class _Mobile(_Azure):
    _type = "mobile"
    _icon_dir = "resources/azure/mobile"


class AppServiceMobile(_Mobile):
    _icon = "app-service---mobile.png"


class MobileEngagement(_Mobile):
    _icon = "mobile-engagement.png"


class NotificationHubs(_Mobile):
    _icon = "notification-hubs.png"


# Aliases



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _GCP


class _Storage(_GCP):
    _type = "storage"
    _icon_dir = "resources/gcp/storage"


class Filestore(_Storage):
    _icon = "filestore.png"


class PersistentDisk(_Storage):
    _icon = "persistent-disk.png"


class Storage(_Storage):
    _icon = "storage.png"


# Aliases

GCS = Storage



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['diagrams', 'diagrams.aws', 'diagrams.azure', 'diagrams.base', 'diagrams.gcp']

package_data = \
{'': ['*']}

install_requires = \
['graphviz>=0.13.2,<0.14.0', 'jinja2>=2.10,<3.0']

setup_kwargs = {
    'name': 'diagrams',
    'version': '0.1.0',
    'description': 'draw cloud system architecture diagrams in code',
    'long_description': '<p align="center">\n\t<img src="assets/img/diagrams.png"/>\n</p>\n\n<h1 align="center">Diagrams</h1>\n<p align="center">\n    Diagram as Code\n</p>\n\nDiagrams lets you to draw the cloud system architectures in Python code.\n\nIt was born for prototyping a new system architecture without any design tools. You can also describe or visualize the existing system architecture as well.\n\n> NOTE: It does not control the actual cloud resources like cloudformation or terraform, but just for drawing the system architecutrre.\n\n`Diagram as Code` allows you to track the architecture diagram changes on any version control system (same as source code tracking)\n\nDiagrams currently supports three major cloud providers: `AWS`, `Azure`, `GCP`.\n\n> Let me know if you are using diagram! I\'ll add you in showcase page. (I\'m working on it!) :)\n\n## Getting Started\n\nIt uses [Graphviz](https://www.graphviz.org/) to render the diagram, so you need to [install Graphviz](https://graphviz.gitlab.io/download/) to use **diagrams**. After installing graphviz (or already have it), install the **diagrams**.\n\n```shell\n$ pip install diagrams\n```\n\nYou can start with [quick start](https://diagram.mingrammer.com/docs/installattion/#quick-start). And you can find [guides](https://diagram.mingrammer.com/diagram) for more details. \n\n## Examples\n\n## ContributingF\n\nTo contribute to diagram, check out [CONTRIBUTING](CONTRIBUTING.md).\n\n## License\n\n[MIT](LICENSE.md)',
    'author': 'mingrammer',
    'author_email': 'mingrammer@gmail.com',
    'url': 'https://diagrams.mingrammer.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# This module is automatically generated by autogen.sh. DO NOT EDIT.

from . import _AWS


class _Migration(_AWS):
    _type = "migration"
    _icon_dir = "resources/aws/migration"


class ApplicationDiscoveryService(_Migration):
    _icon = "application-discovery-service.png"


class CloudendureMigration(_Migration):
    _icon = "cloudendure-migration.png"


class DatabaseMigrationService(_Migration):
    _icon = "database-migration-service.png"


class Datasync(_Migration):
    _icon = "datasync.png"


class MigrationAndTransfer(_Migration):
    _icon = "migration-and-transfer.png"


class MigrationHub(_Migration):
    _icon = "migration-hub.png"


class ServerMigrationService(_Migration):
    _icon = "server-migration-service.png"


class SnowballEdge(_Migration):
    _icon = "snowball-edge.png"


class Snowball(_Migration):
    _icon = "snowball.png"


class Snowmobile(_Migration):
    _icon = "snowmobile.png"


class TransferForSftp(_Migration):
    _icon = "transfer-for-sftp.png"


# Aliases

ADS = ApplicationDiscoveryService
CEM = CloudendureMigration
DMS = DatabaseMigrationService
MAT = MigrationAndTransfer
SMS = ServerMigrationService
